package Opt;

##
##	This is the `Opt.pm' module for reading parameters from command 
##  line and file file.
##
##	Copyright (c) 2001 Jiri Polcar. All rights reserved.
##	<polcar@physics.muni.cz>
##
##	This program is free software; you can redistribute it and/or
##	modify it under GNU GPL.
##
##

$VERSION = "0.2";

use Exporter;
use Getopt::Long;

use strict;

@main::ISA = qw(Exporter);
@main::EXPORT_OK = qw( getOptionsFromCmdLine  
                       getOptionsFromFile 
                       simplifyOptions
                     );

my $true = 1;
my $false = undef;
my $parsing_err = 0;
my $not_a_file = undef;


# read options from command line; 
sub getOptionsFromCmdLine {
	my ( $opt ) = @_;

#
# in `$opt' is already configurated (default, or so)
#

	my @to_get_options;
	my $key;
	my $realy_param;
	my %options;


	@to_get_options = ();

#
# Prepare input parametrs for using `GetOptions()'
#

	foreach $key ( keys( %{$opt} ) ) {
	  push( @to_get_options, $key.$opt->{$key}->{alias}.$opt->{$key}->{param});
	}
	Getopt::Long::Configure ("bundling");
	unless ( GetOptions( \%options, @to_get_options ) ) { return $parsing_err; }
  
  #care about NEGATION
  # it is BOOLEAN option, return TRUE; must care about possible negation

  foreach  $key (  keys %options ) {
    tryToNegate( $opt, $key, \%options );
  }

#
# we have TWO options hash array %{$opt} and %options
# must me "transparent"
#
#	%options	:	siple hash with new values from
#             Getopt::Long::GetOptions
#	%{$opt}  	:	large definition...
#
	updateDefaultsOptions( \%options, $opt );	
	return $true;
}


#	read options ( `$opt' ) from config file `$file'; can read scalar, array and
#	hash value; works only with single lines!!!
sub getOptionsFromFile {
	my ( $file, $opt ) = @_;


	unless ( open( C_FILE, "<$file" ) ) {
		return $not_a_file;
	}
  my $to_parse;
  my $key_word;
  my $o_value;
  my $type;
	my $line;

	my %options;

	while ( $line = <C_FILE> ) {
    chomp( $line ); 
    # discard comments and empty lines
    if ( ($line =~ /^\s*([^\#]*)/) && ( $1 ne '') ) {
			$to_parse = $1;
      # try to get key word and value
      if ( $to_parse =~ /^\s*([^\s=]+)\s*=?\s*(.*)?$/ ) {
        $key_word = $1;
        $o_value = $2;
        # is it realy defined option kwy-word?
			  if ( exists( $opt->{$key_word} ) ) {
          # switch due to TYPE of option: ARRAY, HASH, SCALAR
          $type = ref( $opt->{$key_word}->{value} );

careAboutOption( $opt, $file, $key_word,$o_value,$type,$to_parse,\%options );

			  } else { 
          print STDERR "$file: WARNING: Error  at $. line.\n"; 
          return $parsing_err;
        }
		  }  else { 
        print STDERR "$file: Can not get a KEY WORD on line $..\n"; 
        return $parsing_err;
      } 
	  } # else  { print STDERR "$file: Line $. is a comment or empty.\n"; }
  }
	close( C_FILE );
	updateDefaultsOptions( \%options, $opt );	
	return $true;
}

sub careAboutOption  {
  my ( $opt, $file, $key_word, $o_value, $type, $to_parse, $options) = @_;

  my $value;
  my $key;

# ARRAY

  if ( $type =~ /ARRAY/ )	{
	  unless ( defined( $value = 
             uScalar( $o_value, $key_word, $file, $opt, $options ) ) ) {
      return $parsing_err;
		} else { push( @{$options->{$key_word}}, $value ); }	

# HASH

  } elsif  ( $type =~ /HASH/ )	{
	  ( $key, $value ) = uHash( $o_value, $file, $opt );
		unless ( defined( $key ) && defined( $value ) ) {
      return $parsing_err;
    } else {  ${$options->{$key_word}}{$key} =  $value; }

# SCALAR

  }  else	{ 
    unless ( defined( $options->{$key_word} = 
             uScalar( $o_value, $key_word,  $file, $opt, $options ) ) ) {
      return $parsing_err;
    }
  }

}

#	needy to `getOptionsFromFile'; read scalar value; return UNDEF if
#	unsuccessfull
sub uScalar {
	my ( $o_value, $key_word, $file, $opt, $options ) = @_;

  my $value;
  my $neg;

  # is is BOOLEAN option? 
	if ( $opt->{$key_word}->{param} =~ /\=/ ) {

    # is it a integer option?
	  if ( $opt->{$key_word}->{param} =~ /i/i ) {
			unless ( $o_value =~ /^[-+]?\d+$/) {
				print STDERR "$file: ERROR: Expect integer value at line $..\n";
				return $parsing_err;
			} else { return $o_value; }

    # is it a float option?
		} elsif ( $opt->{$key_word}->{param} =~ /f/i ) {
			unless ( $o_value =~ /^[-+]?\d*\.?\d*([eE][-+]?\d)?\d*$/) {
				print STDERR "$file: ERROR: Expect float value at line $..\n";
				return $parsing_err;
			} else { return $o_value; }

    # it will be a string option
		} else { return $o_value;}

	} else { 
    # it is BOOLEAN option, return TRUE; must care about possible negation
    tryToNegate( $opt, $key_word, $options ); 
  }

}

#	needy to `getOptionsFromFile'; read hash value; return (UNDEF, UNDEF) if
#	unsuccess
sub uHash {
	my ( $o_value, $file, $opt ) = @_;

	my $key = undef; 
	my $value = undef;

  if ( $o_value =~ /^\s*([^\s=]+)\s*=?\s*(.*)?$/ ) {
	  $key =$1;
	  $value = $2;
	}
	unless ( defined( $key) && defined( $value ) ) {
		print STDERR "$file: ERROR: ".
                  "Hash parametr expects KEY and VALUE at line $..\n";
		return (undef, undef);
	} else {
		return ( $key, $value );
	}
}


sub updateDefaultsOptions {
	my ( $options, $opt ) = @_;			#	options	:	siple hash with new values from
                                  #           Getopt::Long::GetOptions
																	#	opt			:	large definition...
	my $key;
	my $tmp;
	my $something;

	foreach $key (keys %{$opt} ) {
# BOOLEAN
		if ( exists( $options->{$key} ) ) {
			$something = $false;
			$_ = ref($options->{$key});

			if ( /HASH/ ) {	
				foreach $tmp ( keys %{$options->{$key}} ) {
          $opt->{$key}->{value}->{$tmp} = $options->{$key}->{$tmp};
        }
			}	elsif ( /ARRAY/ ) {	
        # default value for DUPLICITE
        unless ( defined ( $opt->{$key}->{duplicite} )) { 
          $opt->{$key}->{duplicite} = $false; 
        }
				foreach $tmp ( @{$options->{$key}} ) {
          unless ( $opt->{$key}->{duplicite} ) {
            unless ( isIn( $opt->{$key}->{param} , $tmp , 
                           @{$opt->{$key}->{value}} ) ) {
              push( @{$opt->{$key}->{value}}, $tmp );
            }
          } else {
            push( @{$opt->{$key}->{value}}, $tmp );
          }
        }
			} else {
				$opt->{$key}->{value} = $options->{$key}; 
			}
		} else { next; } 
	}
	return $true;
}


sub isIn {
  my ( $param, $element, @array ) = @_;
  my $tmp;
  foreach $tmp ( @array ) {
    if ( $param =~ /=[if]/ ) {
      if ( $element == $tmp ) { return $true;}
    } else {
      if ( $element eq $tmp ) { return $true;}
    }
  }
  return $false;
}


sub tryToNegate {
  my ( $opt, $key, $options_addr ) = @_;
    
  my $neg;
  my @negs;
  my $tmp;
  if ( exists( $opt->{$key}->{negateto} )) { 
    $neg = $opt->{$key}->{negateto};

    if ( ref($neg) =~ /ARRAY/ ) {
      push (@negs, @{$neg} );
    } else { push (@negs, $neg ); }

    foreach $neg ( @negs ) {
      unless ( exists( $opt->{$neg} ) ) {
        print STDERR "ERROR: Can not negate `".$key."' option!\n";
        return $parsing_err;
      } else {
        # must update directly `$opt', because in `updateDefaultsOptions()'
        # are testes `defined()'
        $options_addr->{$neg} = undef;
        $options_addr->{$key} = $true;
      }
    }
  }
  return $true;
}

sub simplifyOptions {
	my ( $opt ) = @_;

	my $options = {};
	my $key;

	foreach $key ( keys %{$opt}) { $options->{$key} = $opt->{$key}->{value}; }
	return $options;
}



$true;


=cut

=head1 NAME

B<Opt> - Organize configuration.

=head1 SYNOPSIS


  use Opt;     

  use constant CONFIGFILE => $ENV{"HOME"}."/.example";
  use constant VERSION    => "0.1";
  
  $true  = 1;      
  $false = undef;
  
  # care about configuration;
  $cfg = makeConfig( defaultOptions() );
  
  # these shows something and  quit `default'
  if ( $cfg->{'version'} ) { print VERSION,"\n"; exit; }
  if ( $cfg->{'help'} ) { showHelp(); exit; }
  
  # if `--verbose' show brif welcome message
  if ( $cfg->{'verbose'} ) { welcome(); }
  
  
  #...do something.
  print "This is a example for Opt Perl module.\n";
  
  
  ## SUBROUTINES
  
  sub makeConfig {
    my ( $opt ) = @_;
  
    my $file;
  
    $file = $opt->{config}->{value};
    makeConfig_do( $file, $opt );
    if ( $file ne $opt->{config}->{value} ) {
      $file = $opt->{config}->{value};
      makeConfig_do( $file, $opt );    
    }
  
    # simpmlify the %{$opt} HASH; remove `->{value}' level
    return Opt::simplifyOptions( $opt );
  }
  
 
  sub makeConfig_do {
    my ( $file, $opt ) = @_;
  
    my $result;
    my @S = @ARGV;
    local @ARGV = @S;
    $result = Opt::getOptionsFromFile( $file, $opt );
    unless ( defined( $result ) ) { 
      # can not read configfile, so a try to create default
       if ( $file eq CONFIGFILE ) { generateConfigFile(); } else {
        print STDERR "program: Unable to load $file file.\n";
        exit;
      }
    } elsif ( not $result  ) { exit; }
  
    $result = Opt::getOptionsFromCmdLine( $opt );
    unless ( $result ) { exit; }
  
    return $true;
  }
  
  
  # generate default config;
  sub generateConfigFile {
   
   my $cfg = q [
  
  #
  #
  #  Config for `example.pl'
  #
  #
  
  
  ##  use config file; default ~/.example
  #config my_another_config
  
  ##  `example' will be verbose
  verbose
  ];
  
    
    unless ( open(NEWCONFIG, "> ".CONFIGFILE) ) {
      print STDERR "WARNING: Cannot create default config file in ".
                   CONFIGFILE." !\n";
    }
    print NEWCONFIG $cfg;
    close( NEWCONFIG );
  
    return $true;
  }
  
  
  # MAIN deginition for configuration
  sub defaultOptions {
  return {
      'verbose'   =>   { value => $false,
                      param => '',     
                      negateto => 'no-verbose',
                      alias => '|v'
                    },
  
      'no-verbose'   =>   { value => $true,
                      param => '',     
                      negateto => 'verbose',
                      alias => ''
                    },
  
      'help'       =>   { value => $false,
                      param => '',     
                      alias => '|h'
                    },
  
      'version' =>   { value => $false,
                      param => '',     
                      alias => '|V'
                    },
  
      'config'    =>   { value => CONFIGFILE,
                      param => '=s',     
                      alias => ''
                    }
      };
  }
  
  #return C<--help> message
  sub showHelp {
  
    print q[
  Usage: default 
  This is a bref help for `example'.  
  For deatils see default(1) if it exists.
  
  PARAMETERS:
      --config=FILE         use config file FILE (default: ~/.example)
  C<-h>, C<--h>elp                show this help and terminate
  -V, --version             show `example.pl' version and terminate
      --no-verbose          don't be verbose
  -v, --verbose             be verbose
  
    ];
  }
  
  
  # show welcome message
  sub welcome {
    print "example.pl, version " . VERSION . " by Jiri Polcar.\n".
          "This is free software with ABSOLUTELY NO WARRANTY.\n".
          "Run `$0 C<--help>' to get help.\n";
  }
  

=head1 DESCRIPTION

B<Opt> is a Perl module which can help organize work with command line options
and config files.


=head1 AUTHORS

Jiri Polcar E<lt>polcar@physics.muni.cz>

=head1 COPYRIGHT

Copyright (C) 2001 Jiri Polcar <polcar@physics.muni.cz>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

=head1 SEE ALSO

perl(1), GetOpt::Long(pm)

