#!/usr/bin/perl -w

##
##  This is a `example.pl' ( Example for Opt module using. ).
##
##  Copyright (c) 2001 Jiri Polcar. All rights reserved.
##  <polcar@physics.muni.cz>
##
##  This program is free software; you can redistribute it 
##  and/or modify it under GNU GPL.
##
##

use lib qw( ./ );

use Opt;            # module for reading configuration


use constant CONFIGFILE => $ENV{"HOME"}."/.example";
use constant VERSION    => "0.1";

$true  = 1;      
$false = undef;

# main structure for `default'; contains configuraton, ...
my $default;

# care about configuration;
$cfg = makeConfig( defaultOptions() );

# these shows something and  quit `default'
if ( $cfg->{'version'} ) { print VERSION,"\n"; exit; }
if ( $cfg->{'help'} )    { showHelp(); exit; }

# if --verbose show brif welcome message
if ( $cfg->{'verbose'} ) { welcome(); }


#...do something.
print "This is a example for Opt Perl module.\n";
if ( $cfg->{'1'} ) { print "1\n"; }
if ( $cfg->{'2'} ) { print "2\n"; }
if ( $cfg->{'3'} ) { print "3\n"; }


##                       SUBSECTIONS                   ##


# input parametr: default option (large structure); read config from default
# config-file, if non exists create it; read configuration from command line;
# if chadged parametr `config' read THIS file; at the end read again
# commandl line
sub makeConfig {
  my ( $opt ) = @_;

  my $file;

  $file = $opt->{config}->{value};
  makeConfig_do( $file, $opt );
  if ( $file ne $opt->{config}->{value} ) {
    $file = $opt->{config}->{value};
    makeConfig_do( $file, $opt );    
  }

  # simpmlify the %{$opt} HASH; remove `->{value}' level
  return Opt::simplifyOptions( $opt );
}

# subroutine for makeConfig()
sub makeConfig_do {
  my ( $file, $opt ) = @_;

  my $result;
  my @S = @ARGV;
  local @ARGV = @S;
  $result = Opt::getOptionsFromFile( $file, $opt );
  unless ( defined( $result ) ) { 
    # can not read configfile, so a try to create default
     if ( $file eq CONFIGFILE ) { generateConfigFile(); } else {
      print STDERR "program: Unable to load $file file.\n";
      exit;
    }
  } elsif ( not $result  ) { exit; }

  $result = Opt::getOptionsFromCmdLine( $opt );
  unless ( $result ) { exit; }

  return $true;
}


# generate default config;
sub generateConfigFile {
 
 my $cfg = q [

#
#
#  Config for `example.pl'
#
#


##  use config file; default ~/.example
#config my_another_config

##  `example' will be verbose
verbose
];

  
  unless ( open(NEWCONFIG, "> ".CONFIGFILE) ) {
    print STDERR "WARNING: Cannot create default config file in ".
                 CONFIGFILE." !\n";
  }
  print NEWCONFIG $cfg;
  close( NEWCONFIG );

  return $true;
}


# MAIN deginition for configuration
sub defaultOptions {
return {
    'verbose'   =>   { value => $false,
                    param => '',     
                    negateto => [ 'no-verbose' ],
                    alias => '|v'
                  },

    'no-verbose'   =>   { value => $true,
                    param => '',     
                    negateto => 'verbose',
                    alias => ''
                  },

    'help'       =>   { value => $false,
                    param => '',     
                    alias => '|h'
                  },

    'version' =>   { value => $false,
                    param => '',     
                    alias => '|V'
                  },

    '1' =>   { value => $true,
                    param => '',
                    negateto => [ '2', '3' ],     
                    alias => ''
                  },

    '2' =>   { value => $false,
                    param => '',
                    negateto => [ '1', '3' ],     
                    alias => ''
                  },

    '3' =>   { value => $false,
                    param => '',
                    negateto => [ '1', '2' ],     
                    alias => ''
                  },

    'config'    =>   { value => CONFIGFILE,
                    param => '=s',     
                    alias => ''
                  }
    };
}

#return --help message
sub showHelp {

  print
"Usage: default 
This is a bref help for `default'.  For deatils see default(1) if it exists.

PARAMETERS:
    --config=FILE         use config file FILE (default: ~/.elca)
-h, --help                show this help and terminate
-V, --version             show `elca' version and terminate
    --no-verbose          don't be verbose
-v, --verbose             be verbose

";
}


# show welcome message
sub welcome {
  print "example.pl, version " . VERSION . " by Jiri Polcar.\n".
        "This is free software with ABSOLUTELY NO WARRANTY.\n".
        "Run `$0 --help' to get help.\n";
}




